@@ -25,8 +25,7 @@
 #include <glib.h>
 
 #ifndef TOTEM_PL_PARSER_MINI
-#include <zlib.h>
-
+#include <libsoup/soup-gnome.h>
 #include "xmlparser.h"
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
@@ -572,91 +571,6 @@ totem_pl_parser_add_xml_feed (TotemPlParser *parser,
 #endif /* !HAVE_GMIME */
 }
 
-/* From libgsf's gsf-utils.h */
-#define GSF_LE_GET_GUINT32(p)				\
-	(guint32)((((guint8 const *)(p))[0] << 0)  |	\
-		  (((guint8 const *)(p))[1] << 8)  |	\
-		  (((guint8 const *)(p))[2] << 16) |	\
-		  (((guint8 const *)(p))[3] << 24))
-
-/* From libgsf's gsf-input-gzip.c */
-/* gzip flag byte */
-#define GZIP_IS_ASCII		0x01 /* file contains text ? */
-#define GZIP_HEADER_CRC		0x02 /* there is a CRC in the header */
-#define GZIP_EXTRA_FIELD	0x04 /* there is an 'extra' field */
-#define GZIP_ORIGINAL_NAME	0x08 /* the original is stored */
-#define GZIP_HAS_COMMENT	0x10 /* There is a comment in the header */
-#define GZIP_HEADER_FLAGS (unsigned)(GZIP_IS_ASCII |GZIP_HEADER_CRC |GZIP_EXTRA_FIELD |GZIP_ORIGINAL_NAME |GZIP_HAS_COMMENT)
-
-static guint32
-check_header (char *data, gsize len)
-{
-	static guint8 const signature[2] = {0x1f, 0x8b};
-	unsigned flags;
-
-	if (len < 2 + 1 + 1 + 6)
-		return 0;
-
-	/* Check signature */
-	if (memcmp (data, signature, sizeof (signature)) != 0)
-		return 0;
-
-	/* verify flags and compression type */
-	flags  = data[3];
-	if (data[2] != Z_DEFLATED || (flags & ~GZIP_HEADER_FLAGS) != 0)
-		return 0;
-
-	/* Get the uncompressed size */
-	/* FIXME, but how?  The size read here is modulo 2^32.  */
-	return GSF_LE_GET_GUINT32 (data + len - 4);
-}
-
-static char *
-decompress_gzip (char *data, gsize len)
-{
-	guint32 retlen;
-	char *ret;
-	int zerr;
-	z_stream  stream;
-
-	retlen = check_header (data, len);
-	if (retlen == 0)
-		return g_strdup (data);
-
-	stream.zalloc    = (alloc_func)0;
-	stream.zfree     = (free_func)0;
-	stream.opaque    = (voidpf)0;
-	stream.next_in   = Z_NULL;
-	stream.next_out  = Z_NULL;
-	stream.avail_in  = stream.avail_out = 0;
-
-	/* 16 + MAX_WBITS as per http://hewgill.com/journal/entries/349 */
-	if (Z_OK != inflateInit2 (&stream, 16 + MAX_WBITS))
-		return NULL;
-
-	/* +1 so it's NULL-terminated */
-	ret = g_malloc0 (retlen + 1);
-
-	stream.next_in = (unsigned char *) data;
-	stream.avail_in = len;
-
-	stream.next_out = (unsigned char *) ret;
-	stream.avail_out = retlen;
-
-	zerr = inflate (&stream, Z_NO_FLUSH);
-	if (zerr != Z_OK && zerr != Z_STREAM_END) {
-		g_free (ret);
-		return NULL;
-	}
-	zerr = inflateEnd (&stream);
-	if (zerr != Z_OK) {
-		g_free (ret);
-		return NULL;
-	}
-
-	return ret;
-}
-
 static const char *
 totem_pl_parser_parse_itms_doc (xml_node_t *item)
 {
@@ -713,26 +627,34 @@ totem_pl_parser_get_feed_uri (char *data, gsize len)
 	return ret;
 }
 
-static char *
-totem_pl_parser_get_itms_uri (const char *data)
+static GByteArray *
+totem_pl_parser_load_http_itunes (const char *uri)
 {
-	char *s, *end, *ret;
-#define ITMS_OPEN "<body onload=\"return itmsOpen('"
+	SoupMessage *msg;
+	SoupSession *session;
+	GByteArray *data;
 
-	/* The bit of text looks like:
-	 * <body onload="return itmsOpen('itms://ax.phobos.apple.com.edgesuite.net/WebObjects/MZStore.woa/wa/viewPodcast?id=207870198&amp;ign-mscache=1','http://www.apple.com/uk/itunes/affiliates/download/?itmsUrl=itms%3A%2F%2Fax.phobos.apple.com.edgesuite.net%2FWebObjects%2FMZStore.woa%2Fwa%2FviewPodcast%3Fid%3D207870198%26ign-mscache%3D1','userOverridePanel',false)"> */
+	session = soup_session_sync_new_with_options (
+	    SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_GNOME_FEATURES_2_26,
+	    SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_CONTENT_DECODER,
+	    SOUP_SESSION_USER_AGENT, "iTunes/7.4.1",
+	    SOUP_SESSION_ACCEPT_LANGUAGE_AUTO, TRUE,
+	    NULL);
 
-	s = strstr (data, ITMS_OPEN);
-	if (s == NULL)
-		return NULL;
-	s = s + strlen (ITMS_OPEN);
-	end = strchr (s, '\'');
-	if (end == NULL)
+	msg = soup_message_new (SOUP_METHOD_GET, uri);
+	soup_session_send_message (session, msg);
+	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		data = g_byte_array_new ();
+		g_byte_array_append (data,
+				     (guchar *) msg->response_body->data,
+				     msg->response_body->length);
+	} else {
 		return NULL;
+	}
+	g_object_unref (msg);
+	g_object_unref (session);
 
-	ret = g_strndup (s, end - s);
-	memcpy (ret, "http", 4);
-	return ret;
+	return data;
 }
 
 TotemPlParserResult
@@ -745,57 +667,28 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 #ifndef HAVE_GMIME
 	WARN_NO_GMIME;
 #else
-	char *contents, *uncompressed, *itms_uri;
-	GFile *itms_file, *feed_file;
+	GByteArray *content;
+	char *itms_uri;
+	GFile *feed_file;
 	TotemPlParserResult ret;
-	gsize size;
 
 	if (g_file_has_uri_scheme (file, "itms") == FALSE) {
-		/* Get the webpage */
-		if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE) {
-			DEBUG(file, g_print ("Couldn't load contents for %s\n", uri));
-			return TOTEM_PL_PARSER_RESULT_ERROR;
-		}
-
-		uncompressed = decompress_gzip (contents, size);
-		g_free (contents);
-		if (uncompressed == NULL) {
-			DEBUG(file, g_print ("Couldn't decompress %s\n", uri));
-			return TOTEM_PL_PARSER_RESULT_ERROR;
-		}
-
-		/* Look for the link to the itms on phobos */
-		itms_uri = totem_pl_parser_get_itms_uri (uncompressed);
-		g_free (uncompressed);
-	} else {
-		itms_uri= g_file_get_uri (file);
-		memcpy (itms_uri, "http", 4);
-	}
-
-	/* Get the phobos linked, in some weird iTunes only format */
-	itms_file = g_file_new_for_uri (itms_uri);
-	g_free (itms_uri);
-
-	if (g_file_load_contents (itms_file, NULL, &contents, &size, NULL, NULL) == FALSE) {
-		DEBUG(itms_file, g_print ("Couldn't load contents for itms_file %s\n", uri));
-		g_object_unref (itms_file);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
-	g_object_unref (itms_file);
+	itms_uri= g_file_get_uri (file);
+	memcpy (itms_uri, "http", 4);
 
-	uncompressed = decompress_gzip (contents, size);
-	g_free (contents);
-	if (uncompressed == NULL) {
-		DEBUG(itms_file, g_print ("Couldn't decompress itms_file %s\n", uri));
+	if (itms_uri == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
-	}
+
+	/* Load the file using iTunes user-agent */
+	content = totem_pl_parser_load_http_itunes (itms_uri);
 
 	/* And look in the file for the feedURL */
-	feed_file = totem_pl_parser_get_feed_uri (uncompressed, strlen (uncompressed) + 1);
-	if (feed_file == NULL) {
-		g_free (uncompressed);
+	feed_file = totem_pl_parser_get_feed_uri ((char *) content->data, content->len);
+	g_byte_array_free (content, TRUE);
+	if (feed_file == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
-	}
 
 	DEBUG(feed_file, g_print ("Found feed URI: %s\n", uri));
 
@@ -816,26 +709,12 @@ totem_pl_parser_is_itms_feed (GFile *file)
 	uri = g_file_get_uri (file);
 
 	if (g_file_has_uri_scheme (file, "itms") != FALSE) {
-		if (strstr (uri, "phobos.apple.com") != NULL ||
-		    strstr (uri, "itunes.apple.com") != NULL) {
-			if (strstr (uri, "viewPodcast") != NULL) {
-				g_free (uri);
-				return TRUE;
-			}
+		if (strstr (uri, "/podcast/") != NULL) {
+			g_free (uri);
+			return TRUE;
 		}
 	}
 
-	if (strstr (uri, "phobos.apple.com/") != NULL
-	    && strstr (uri, "viewPodcast") != NULL) {
-	    	g_free (uri);
-		return TRUE;
-	}
-
-	if (strstr (uri, "itunes.com/podcast") != NULL) {
-		g_free (uri);
-		return TRUE;
-	}
-
 	g_free (uri);
 
 	return FALSE;
